
//
//  Sampling 'object' encapsulating a CARAS sampler data
//
//      sampler_data is raw data
//         => low level infos
//
//      sampling_p is a sampler including range and trigger infos
//         => higher level infos
//         => kind of oriented object
//

#ifndef SAMPLING_H
#define SAMPLING_H 1

#ifdef __cplusplus
extern "C" {
#endif

#include <string.h>
#include <stdio.h>

#include "fasterac/fasterac.h"
#include "fasterac/sampler.h"

   //  type sampling_t

   typedef struct sampling_t {
      //  BASICS
      unsigned short    label;
      //long double       ts_ns;
      double            ts_ns;
      //  META
      sampler_range_t   range_mV;
      double            before_th_ns;
      //  DATA (mV)
      short             data [SAMPLER_NB_PTS_MAX];
      unsigned short    nb_pts;
      //  USEFULL VALUES
      double            max_mV;
      double            min_mV;
      double            qtot_mVns;
      //  TIMES RELATIVE TO TRIGGER (0 ns)
      double            first_ns;
      double            last_ns;
      double            max_ns;
      double            min_ns;
   } sampling_t;

   //  initialization

   int               sampling_init         (sampling_t*     s,        //  return 0 when ok
                                            faster_data_p   sampler,
                                            sampler_range_t range_mV,
                                            double          duration_before_threshold_ns);

   //  data basics

   unsigned short    sampling_label        (sampling_t const* s);
   //long double       sampling_clock_ns     (sampling_t const* s);       // time stamp (lsb = 2ns)
   double            sampling_clock_ns     (sampling_t const* s);       // time stamp (lsb = 2ns)
   unsigned short    sampling_nb_pts       (sampling_t const* s);

   //  usefull values

   double            sampling_first_ns     (sampling_t const* s);
   double            sampling_last_ns      (sampling_t const* s);

   double            sampling_max_mV       (sampling_t const* s);
   double            sampling_max_ns       (sampling_t const* s);

   double            sampling_min_mV       (sampling_t const* s);
   double            sampling_min_ns       (sampling_t const* s);

   double            sampling_qtot_mVns    (sampling_t const* s);

   double            sampling_value_mV     (sampling_t const* s,
                                            double            at_ns);   //  first_ns <= at_ns <= last_ns

   double            sampling_average_mV   (sampling_t const* s,
                                            double            from_ns,
                                            double            to_ns);

   double            sampling_charge_mVns  (sampling_t const* s,
                                            double            from_ns,
                                            double            to_ns);

   double            sampling_xpoint_ns    (sampling_t const* s,            //  returns the crossing point at xval_mV (basic interpolation)
                                            double            xval_mV,      //  this time is referenced by the 0ns of the sampling (acquisition threshold)
                                            int               rising_edge,  //  direction of crossing (1 -> rising edge | 0 -> falling edge)
                                            double            during_ns,    //  crossing min duration, no back crossing before that time, value min is 2ns
                                            double            from_ns,      //  start of crossing search
                                            double            to_ns);       //  stop of crossing search
                                                                            //  returns a value after 'to_ns' if there is no crossing between from and to
   //  conversions

   unsigned int      sampling_to_idx       (sampling_t const* s,
                                            double            at_ns);  //  first_ns <= at_ns <= last_ns

   double            sampling_to_ns        (sampling_t const* s,
                                            unsigned int      idx);    //  0 <= idx <= nb_pts - 1

   double            sampling_to_mV        (sampling_t const* s,
                                            short             rawdata);

   //

#ifdef __cplusplus
}
#endif


#endif  // SAMPLING_H
